@@ -5,7 +5,7 @@
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
-# Foundation; either version 2 of the License, or (at your option) any later
+# Foundation; either version 3 of the License, or (at your option) any later
 # version.
 #
 # This program is distributed in the hope that it will be useful, but WITHOUT
@@ -18,12 +18,13 @@
 #
 ###############################################################################
 
+import os
+import sys
+
+from genshi.template import TemplateLoader
+
 __metaclass__ = type
 
-import os, sys
-
-import pkg_resources
-from genshi.template import TemplateLoader
 
 def _absolute(path):
     "Compute the absolute path of path relative to the caller file"
@@ -33,6 +34,7 @@ def _absolute(path):
     caller_dir = os.path.dirname(caller_fname)
     return os.path.abspath(os.path.join(caller_dir, path))
 
+
 def _guess_type(mime):
     """
     Returns the codename used by relatorio to identify which template plugin
@@ -124,7 +126,6 @@ class Report:
         return '<relatorio report on %s>' % self.fpath
 
 
-
 class ReportDict:
 
     def __init__(self, *args, **kwargs):
