@@ -1,6 +1,6 @@
 ###############################################################################
 #
-# Copyright (c) 2009-2014 Cedric Krier.
+# Copyright (c) 2009-2017 Cedric Krier.
 # Copyright (c) 2007, 2008 OpenHex SPRL. (http://openhex.com) All Rights
 # Reserved.
 #
@@ -19,8 +19,6 @@
 #
 ###############################################################################
 
-__metaclass__ = type
-
 import re
 try:
     # requires python 2.5+
@@ -28,6 +26,8 @@ try:
 except ImportError:
     from md5 import md5
 
+import base64
+import mimetypes
 import time
 import urllib
 import zipfile
@@ -38,7 +38,6 @@ from decimal import Decimal
 
 
 import warnings
-warnings.filterwarnings('always', module='relatorio.templates.opendocument')
 
 import lxml.etree
 import genshi
@@ -57,6 +56,9 @@ try:
 except ImportError:
     ChartTemplate = type(None)
 
+__metaclass__ = type
+warnings.filterwarnings('always', module='relatorio.templates.opendocument')
+
 GENSHI_EXPR = re.compile(r'''
         (/)?                                 # is this a closing tag?
         (for|if|choose|when|otherwise|with|
@@ -74,14 +76,6 @@ GENSHI_CLOSING_DIRECTIVE = ['for',
                             'with',
                             ]
 
-EXTENSIONS = {'image/png': 'png',
-              'image/jpeg': 'jpg',
-              'image/bmp': 'bmp',
-              'image/gif': 'gif',
-              'image/tiff': 'tif',
-              'image/xbm': 'xbm',
-             }
-
 RELATORIO_URI = 'relatorio'
 GENSHI_URI = 'http://genshi.edgewall.org/'
 MANIFEST = 'META-INF/manifest.xml'
@@ -109,12 +103,6 @@ EtreeElement = lxml.etree.Element
 # parent node defines it. Unfortunately, lxml doesn't support this:
 # the nsmap attribute of Element objects is (currently) readonly.
 
-def guess_type(val):
-    if isinstance(val, (str, unicode)):
-        return 'string'
-    elif isinstance(val, (int, float, long, Decimal)):
-        return 'float'
-
 
 class OOTemplateError(genshi.template.base.TemplateSyntaxError):
     "Error to raise when there is a SyntaxError in the genshi template"
@@ -139,7 +127,8 @@ class ImageHref:
         bitstream.seek(0)
         file_content = bitstream.read()
         name = md5(file_content).hexdigest()
-        path = 'Pictures/%s.%s' % (name, EXTENSIONS[mimetype])
+        path = 'Pictures/%s%s' % (
+            name, mimetypes.guess_extension(mimetype))
         if path not in self.zip.namelist():
             self.zip.writestr(path, file_content)
             self.manifest.add_file_entry(path, mimetype)
@@ -222,6 +211,26 @@ def wrap_nodes_between(first, last, new_parent):
     old_parent.remove(last)
 
 
+def remove_node_keeping_tail(node):
+    """Remove the node from the tree but keeping tail by appending to the
+    previous or parent node.
+    """
+    parent = node.getparent()
+    if node.tail:
+        previous = node.getprevious()
+        if previous is not None:
+            if not previous.tail:
+                previous.tail = node.tail
+            else:
+                previous.tail += node.tail
+        else:
+            if not parent.text:
+                parent.text = node.tail
+            else:
+                parent.text += node.tail
+    parent.remove(node)
+
+
 def update_py_attrs(node, value):
     """An helper function to update py_attrs of a node.
     """
@@ -266,7 +275,7 @@ class Template(MarkupTemplate):
             source = self.filepath
         self._source = source
         self.filepath = None  # Prevent zip content in traceback
-        zf = zipfile.ZipFile(source)
+        zf = get_zip_file(source)
         content = zf.read('content.xml')
         styles = zf.read('styles.xml')
 
@@ -322,6 +331,7 @@ class Template(MarkupTemplate):
         self.namespaces['py'] = GENSHI_URI
         self.namespaces['relatorio'] = RELATORIO_URI
 
+        self._remove_soft_page_break(tree)
         self._invert_style(tree)
         self._handle_relatorio_tags(tree)
         self._handle_images(tree)
@@ -329,6 +339,17 @@ class Template(MarkupTemplate):
         self._escape_values(tree)
         return BytesIO(lxml.etree.tostring(tree))
 
+    def _remove_soft_page_break(self, tree):
+        "remove soft-page-break tag and use-soft-page-break attribute"
+        xpath_expr = "//text:soft-page-break"
+        for node in tree.xpath(xpath_expr, namespaces=self.namespaces):
+            remove_node_keeping_tail(node)
+
+        xpath_expr = "//office:text[@text:use-soft-page-breaks]"
+        text_namespace = self.namespaces['text']
+        for node in tree.xpath(xpath_expr, namespaces=self.namespaces):
+            node.attrib.pop('{%s}use-soft-page-breaks' % text_namespace)
+
     def _invert_style(self, tree):
         "inverts the text:a and text:span"
         xpath_expr = "//text:a[starts-with(@xlink:href, 'relatorio://')]" \
@@ -407,8 +428,11 @@ class Template(MarkupTemplate):
         table_namespace = self.namespaces['table']
         table_row_tag = '{%s}table-row' % table_namespace
         table_cell_tag = '{%s}table-cell' % table_namespace
+        text_namespace = self.namespaces['text']
+        text_style_attributes = [s % text_namespace for s in [
+                '{%s}class-names', '{%s}cond-style-name', '{%s}style-name']]
 
-        office_name = '{%s}value' % self.namespaces['office']
+        office_value = '{%s}value' % self.namespaces['office']
         office_valuetype = '{%s}value-type' % self.namespaces['office']
         if 'calcext' in self.namespaces:
             calcext_valuetype = '{%s}value-type' % self.namespaces['calcext']
@@ -491,9 +515,17 @@ class Template(MarkupTemplate):
                 # remove the directive node
                 r_node.getparent().remove(r_node)
             else:
+                def has_style(node):
+                    return any(attr in node.attrib
+                               for attr in text_style_attributes)
                 # It's not a genshi statement it's a python expression
-                parent = r_node.getparent().getparent()
-                if parent is None or parent.tag != table_cell_tag:
+                parent = r_node.getparent()
+                grand_parent = parent.getparent()
+                # Guess type only if it is the only value in the cell
+                # and its parent has no style
+                if (grand_parent is None
+                        or grand_parent.tag != table_cell_tag
+                        ) or len(grand_parent) != 1 or has_style(parent):
                     r_node.attrib[py_replace] = expr
                     continue
 
@@ -502,20 +534,14 @@ class Template(MarkupTemplate):
                                              cache_id)
                 # The grand-parent tag is a table cell we should set the
                 # correct value and type for this cell.
-                dico = ("{'%s': __relatorio_store_cache(%s, %s), "
-                        "'%s': __relatorio_guess_type("
-                        "__relatorio_get_cache(%s))}")
-                update_py_attrs(parent, dico %
-                        (office_name, cache_id, expr, office_valuetype,
-                         cache_id))
-                parent.attrib.pop(office_valuetype, None)
-                parent.attrib.pop(office_name, None)
-                if (calcext_valuetype and
-                        parent.attrib.pop(calcext_valuetype, None)):
-                    update_py_attrs(parent,
-                        "{'%s': __relatorio_guess_type("
-                        "__relatorio_get_cache(%s))}" %
-                        (calcext_valuetype, cache_id))
+                dico = ('__relatorio_guess_type('
+                        '__relatorio_store_cache(%s, %s))')
+                update_py_attrs(grand_parent, dico % (cache_id, expr))
+                for attr in [office_value,
+                             office_valuetype,
+                             calcext_valuetype]:
+                    if attr:
+                        grand_parent.attrib.pop(attr, None)
 
     def _handle_column_loops(self, statement, ancestor, opening,
                              outer_o_node, outer_c_node):
@@ -741,6 +767,41 @@ class Template(MarkupTemplate):
             if element.text:
                 element.text = element.text.replace(PREFIX, PREFIX * 2)
 
+    def _guess_type(self, val):
+        office_namespace = self.namespaces['office']
+        types = {'boolean': '{%s}boolean-value' % office_namespace,
+                 'currency': '{%s}currency' % office_namespace,
+                 'date': '{%s}date-value' % office_namespace,
+                 'float': '{%s}value' % office_namespace,
+                 'percentage': '{%s}value' % office_namespace,
+                 'string': '{%s}string-value' % office_namespace,
+                 'time': '{%s}time-value' % office_namespace,
+                 'void': '{%s}value' % office_namespace,
+                 }
+        attrs = dict.fromkeys(types)
+        # Missing base type for currency and percentage
+        if isinstance(val, bool):
+            type_ = 'boolean'
+            val = str(val).lower()
+        elif isinstance(val, datetime.date):
+            type_ = 'date'
+            val = val.isoformat()
+        elif isinstance(val, (int, float, long, Decimal)):
+            type_ = 'float'
+        elif isinstance(val, basestring):
+            type_ = 'string'
+        elif isinstance(val, datetime.timedelta):
+            type_ = 'time'
+            val = 'P%sD%sS' % (val.days, val.seconds)
+        else:
+            type_ = 'void'
+            val = None
+        attrs[types[type_]] = val
+        attrs['{%s}value-type' % office_namespace] = type_
+        if 'calcext' in self.namespaces:
+            attrs['{%s}value-type' % self.namespaces['calcext']] = type_
+        return attrs
+
     def generate(self, *args, **kwargs):
         "creates the RelatorioStream."
         serializer = OOSerializer(self._source)
@@ -748,7 +809,7 @@ class Template(MarkupTemplate):
                                                     serializer.manifest,
                                                     kwargs)
         kwargs['__relatorio_make_dimension'] = ImageDimension(self.namespaces)
-        kwargs['__relatorio_guess_type'] = guess_type
+        kwargs['__relatorio_guess_type'] = self._guess_type
 
         counter = ColumnCounter()
         kwargs['__relatorio_reset_col_count'] = counter.reset
@@ -800,6 +861,86 @@ class DuplicateColumnHeaders(object):
                 yield mark, (kind, data, pos)
 
 
+def get_zip_file(source):
+    try:
+        return zipfile.ZipFile(source)
+    except zipfile.BadZipfile:
+        # ZipFile modify the position
+        if hasattr(source, 'seek'):
+            source.seek(0)
+        return zipfile.ZipFile(fod2od(source))
+
+
+def fod2od(source):
+    "Convert Flat OpenDocument to OpenDocument"
+    odt_io = BytesIO()
+    odt_zip = zipfile.ZipFile(odt_io, mode='w')
+    fodt_tree = lxml.etree.parse(source)
+    fodt_root = fodt_tree.getroot()
+    office_ns = fodt_root.nsmap['office']
+    tag2files = {
+        '{%s}meta' % office_ns: ['meta'],
+        '{%s}settings' % office_ns: ['settings'],
+        '{%s}scripts' % office_ns: ['content'],
+        '{%s}font-face-decls' % office_ns: ['content', 'styles'],
+        '{%s}styles' % office_ns: ['styles'],
+        '{%s}automatic-styles' % office_ns: ['content', 'styles'],
+        '{%s}master-styles' % office_ns: ['styles'],
+        '{%s}body' % office_ns: ['content'],
+        }
+    mimetype = fodt_root.attrib['{%s}mimetype' % office_ns]
+    documents = {}
+    images = []
+    for child in fodt_root:
+        for fname in tag2files[child.tag]:
+            document = documents.get(fname)
+            if document is None:
+                document = lxml.etree.Element(
+                    '{%s}document-%s' % (office_ns, fname),
+                    nsmap=fodt_root.nsmap)
+                documents[fname] = document
+            child = deepcopy(child)
+            images.extend(extract_images(
+                    child, fodt_root.nsmap, start=len(images)))
+            document.append(child)
+    manifest = Manifest(b'''<?xml version="1.0" encoding="UTF-8"?>
+        <manifest:manifest
+        xmlns:manifest="urn:oasis:names:tc:opendocument:xmlns:manifest:1.0"/>
+        ''')
+    manifest.add_file_entry('/', mimetype)
+    for fname, document in documents.iteritems():
+        document_string = lxml.etree.tostring(document, encoding='UTF-8',
+                                              xml_declaration=True)
+        odt_zip.writestr('%s.xml' % fname, document_string)
+        manifest.add_file_entry('%s.xml' % fname, 'text/xml')
+    for fname, data, mime_type in images:
+        odt_zip.writestr(fname, data)
+        manifest.add_file_entry(fname, mime_type)
+    odt_zip.writestr(MANIFEST, str(manifest))
+    odt_zip.writestr('mimetype', mimetype)
+    odt_zip.close()
+    return odt_io
+
+
+def extract_images(child, namespaces, start=0):
+    "Extract draw:image with binary-data and replace by href"
+    import magic
+    images = []
+    for i, image in enumerate(
+            child.xpath('//draw:image', namespaces=namespaces), start):
+        binary_data, = image.xpath(
+            './office:binary-data', namespaces=namespaces)
+        data = base64.b64decode(binary_data.text)
+        mime_type = magic.from_buffer(data, mime=True)
+        name = 'Pictures/image%s%s' % (
+            i, mimetypes.guess_extension(mime_type))
+        image.remove(binary_data)
+        xlink_ns = namespaces['xlink']
+        image.attrib['{%s}href' % xlink_ns] = name
+        images.append((name, data, mime_type))
+    return images
+
+
 class Manifest(object):
 
     def __init__(self, content):
@@ -882,7 +1023,7 @@ class Meta(object):
 class OOSerializer:
 
     def __init__(self, source):
-        self.inzip = zipfile.ZipFile(source)
+        self.inzip = get_zip_file(source)
         self.manifest = Manifest(self.inzip.read(MANIFEST))
         self.meta = Meta(self.inzip.read(META))
         self.new_oo = BytesIO()
