@@ -1,7 +1,7 @@
 # -*- encoding: utf-8 -*-
 ###############################################################################
 #
-# Copyright (c) 2014 Cedric Krier.
+# Copyright (c) 2014-2017 Cedric Krier.
 # Copyright (c) 2007, 2008 OpenHex SPRL. (http://openhex.com) All Rights
 # Reserved.
 #
@@ -23,7 +23,7 @@
 
 import os
 import unittest
-from io import StringIO
+from io import StringIO, BytesIO
 
 import lxml.etree
 from genshi.filters import Translator
@@ -31,7 +31,7 @@ from genshi.core import PI
 from genshi.template.eval import UndefinedError
 
 from relatorio.templates.opendocument import Template, GENSHI_EXPR,\
-    GENSHI_URI, RELATORIO_URI
+    GENSHI_URI, RELATORIO_URI, fod2od, remove_node_keeping_tail
 
 OO_TABLE_NS = "urn:oasis:names:tc:opendocument:xmlns:table:1.0"
 
@@ -59,7 +59,8 @@ class TestOOTemplating(unittest.TestCase):
     def setUp(self):
         thisdir = os.path.dirname(__file__)
         filepath = os.path.join(thisdir, 'test.odt')
-        self.oot = Template(open(filepath, mode='rb'))
+        self._source = open(filepath, mode='rb')
+        self.oot = Template(self._source)
         self.data = {'first_name': u'Trente',
                      'last_name': u'Møller',
                      'ville': u'Liège',
@@ -76,6 +77,12 @@ class TestOOTemplating(unittest.TestCase):
                      'oeuf': open(os.path.join(thisdir, 'egg.jpg'), 'rb'),
                      'footer': u'We sell stuff'}
 
+    def tearDown(self):
+        self._source.close()
+        for image in self.data['images']:
+            image[0].close()
+        self.data['oeuf'].close()
+
     def test_init(self):
         "Testing the correct handling of the styles.xml and content.xml files"
         self.assertTrue(isinstance(self.oot.stream, list))
@@ -318,3 +325,90 @@ class TestOOTemplating(unittest.TestCase):
         # another non matching expr
         group = GENSHI_EXPR.match('formatLang("en")').groups()
         self.assertEqual(group, (None, None, None, None))
+
+    def test_fodt2odt(self):
+        "Testing converter fodt to odt"
+        thisdir = os.path.dirname(__file__)
+        filepath = os.path.join(thisdir, 'test.fodt')
+        fod2od(filepath)
+
+    def test_generate_fod(self):
+        "Testing generate fod"
+        thisdir = os.path.dirname(__file__)
+        filepath = os.path.join(thisdir, 'test.fodt')
+        with open(filepath, mode='rb') as source:
+            oot = Template(source)
+            oot.generate(**self.data)
+
+
+class TestRemoveNodeKeepingTail(unittest.TestCase):
+
+    def test_without_tail(self):
+        "Testing remove_node_keeping_tail without tail"
+        xml = b'''<parent><target/></parent>'''
+        tree = lxml.etree.parse(BytesIO(xml))
+        target = tree.getroot()[0]
+
+        remove_node_keeping_tail(target)
+
+        self.assertEqual(lxml.etree.tostring(tree), b'''<parent/>''')
+
+    def test_with_tail(self):
+        "Testing remove_node_keeping_tail with tail"
+        xml = b'''<parent><target/>tail</parent>'''
+        tree = lxml.etree.parse(BytesIO(xml))
+        target = tree.getroot()[0]
+
+        remove_node_keeping_tail(target)
+
+        self.assertEqual(
+            lxml.etree.tostring(tree),
+            b'''<parent>tail</parent>''')
+
+    def test_with_tail_and_parent_text(self):
+        "Testing remove_node_keeping_tail with tail and parent text"
+        xml = b'''<parent>text<target/>tail</parent>'''
+        tree = lxml.etree.parse(BytesIO(xml))
+        target = tree.getroot()[0]
+
+        remove_node_keeping_tail(target)
+
+        self.assertEqual(
+            lxml.etree.tostring(tree),
+            b'''<parent>texttail</parent>''')
+
+    def test_without_tail_and_with_previous(self):
+        "Testing remove_node_keeping_tail without tail and with previous"
+        xml = b'''<parent><previous/><target/></parent>'''
+        tree = lxml.etree.parse(BytesIO(xml))
+        target = tree.getroot()[1]
+
+        remove_node_keeping_tail(target)
+
+        self.assertEqual(
+            lxml.etree.tostring(tree),
+            b'''<parent><previous/></parent>''')
+
+    def test_with_tail_and_previous(self):
+        "Testing remove_node_keeping_tail with tail and previous"
+        xml = b'''<parent><previous/><target/>tail</parent>'''
+        tree = lxml.etree.parse(BytesIO(xml))
+        target = tree.getroot()[1]
+
+        remove_node_keeping_tail(target)
+
+        self.assertEqual(
+            lxml.etree.tostring(tree),
+            b'''<parent><previous/>tail</parent>''')
+
+    def test_with_tail_and_previous_tail(self):
+        "Testing remove_node_keeping_tail with tail and previous tail"
+        xml = b'''<parent><previous/>tail<target/>tail</parent>'''
+        tree = lxml.etree.parse(BytesIO(xml))
+        target = tree.getroot()[1]
+
+        remove_node_keeping_tail(target)
+
+        self.assertEqual(
+            lxml.etree.tostring(tree),
+            b'''<parent><previous/>tailtail</parent>''')
